// DigIO.h : main header file for the DigIO DLL
//

#if !defined(AFX_DigIO_H__65E6C3BC_87B9_11D3_8080_00105A1E5D53__INCLUDED_)
#define AFX_DigIO_H__65E6C3BC_87B9_11D3_8080_00105A1E5D53__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#ifndef __AFXWIN_H__
#error include 'stdafx.h' before including this file for PCH
#endif

#include "Resource.h"
#include "I2CMgr.h"
#include "I2CDeviceClass.h"
#include "Module.h"
#include "Xpert/Win32Fix.h"

#undef DECL
#ifdef ENGINE_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif

const int NUM_DIGITALCHANNELS = 8;

// CDigitalChannel is a class that seemlessly deals with converting a 1-8 channel
// being passed in to this module which uses 0-7 numbering.
//
class CDigitalChannel
{
public:
   CDigitalChannel(int nChannel)                   { Set(nChannel);                                      };
   CDigitalChannel()                               { Channel = 0; Valid = false;                         };
   CDigitalChannel(TProperty& Property)            { Set(Property.AsInteger());                          }; 
   CDigitalChannel(TValue& Value)                  { Set(Value.AsInteger());                             };
   CDigitalChannel(const CDigitalChannel& dc)      { Channel = dc.Channel; Valid=dc.Valid;               };
   CDigitalChannel& operator=(CDigitalChannel dc)  { Channel = dc.Channel; Valid=dc.Valid; return *this; };
   CDigitalChannel& operator=(int nChannel)        { Set(nChannel); return *this;                        };
   CDigitalChannel& operator=(TProperty& Property) { Set(Property.AsInteger()); return *this;            };
   CDigitalChannel& operator=(TValue& Value)       { Set(Value.AsInteger()); return *this;               };
   operator BYTE() const                           { return Channel;                                     };
   bool IsValid() const                            { return Valid;                                       };
private:
   void Set(int nChannel)                          { Channel = (Valid = (1 <= nChannel) && (nChannel <= NUM_DIGITALCHANNELS)) ? nChannel - 1 : 0; };
   BYTE Channel;
   bool Valid;
};


/**********************************************************************************************/

//                                        DIGITAL IO

/**********************************************************************************************/

// UNRESOLVED - error codes

union IOCONFIGURATION
{
   struct
   {
      BYTE UpdateMask;
      BYTE ShaftMask;
      BYTE InvertMask;
      BYTE ChangeMask;
      BYTE AlarmMask;
      BYTE InputMask;
      BYTE OutputData;
      BYTE AlarmThreshold0;
      BYTE AlarmThreshold1;
      BYTE AlarmThreshold2;
      BYTE AlarmThreshold3;
      BYTE AlarmThreshold4;
      BYTE AlarmThreshold5;
      BYTE AlarmThreshold6;
      BYTE AlarmThreshold7;
      BYTE FilterTimeConstant0;
      BYTE FilterTimeConstant1;
      BYTE FilterTimeConstant2;
      BYTE FilterTimeConstant3;
      BYTE FilterTimeConstant4;
      BYTE FilterTimeConstant5;
      BYTE FilterTimeConstant6;
      BYTE FilterTimeConstant7;
      BYTE InputThreshold;
   };
   BYTE ConfigurationArray[24];
};
// these constants used to offset into IOCONFIGURATION
#define IOCONFIG_SIZE_OF_CONFIG         15      // this is not sizeof(IOCONFIGURATION) but number of bytes that get sent with a CmdConfigure
#define IOCONFIG_SIZE_OF_FILTERS        8       // number of bytes to be sent with a CmdConfigureFilters
#define IOCONFIG_ALARM_OFFSET           7
#define IOCONFIG_FILTER_OFFSET          15


struct PULSECONFIGURATION
{
   BYTE UpdateMask;
   BYTE Data;
};


class DECL DigitalIO_API : public IODevice
{
public:
   DigitalIO_API(BYTE AddrOfDevice = 0x04, BYTE AddrOfI2C = 0xFE);
   ~DigitalIO_API();

public:
   // the commands are specific to the Digital IO
   I2CCODE CmdConfigure(IOCONFIGURATION& Data) {return Tx(CONFIGURE_OPCODE, Data.ConfigurationArray, IOCONFIG_SIZE_OF_CONFIG);};
   I2CCODE CmdReadData(I2CMSG& MsgResponse, BYTE OpCode);
   I2CCODE CmdLoadCounter(const CDigitalChannel& Channel, UINT32 CounterValue);
   I2CCODE CmdConfigureFilters(IOCONFIGURATION& Data);
   I2CCODE CmdSetSensitivity(BYTE Sensitivity) {return Tx(SETSENSITIVITY_OPCODE, (void *)&Sensitivity, 1);};
   I2CCODE CmdSetSpeed(UINT16 SamplingInterval);    // sets the number of 1/0x8000 seconds in a sampling interval
   I2CCODE CmdPulseOut(BYTE UpdateMask, BYTE Data, UINT16 PulseWidth);

public:
   // higher level
   // the functions now automatically call CmdReadData depending on the DataRefreshInterval
   I2CCODE ReadCount(const CDigitalChannel& Channel, UINT32& CountValue);   // reads the count of the channel
   I2CCODE ReadTime(UINT32& TimeValue);   // interprets the time from the data in a received packet
   I2CCODE ReadCountAndTime(const CDigitalChannel& Channel, UINT32& CountValue, UINT32& TimeValue);   // interprets the count and time from the data in a received packet
   I2CCODE ReadFilteredInputDataBits(const CDigitalChannel& Channel, BOOL& Data);// reads & returns input state of channel
   I2CCODE ReadAllFilteredInputDataBits(int& Data); // read all the inputs
public:
   // these functions handle requests to start and stop sampling, and also AuxOn/Off
   bool MasterStop(void);       // overriden from base class

private:
   UINT16 SamplingSpeed;
public:
   I2CCODE SetSamplingSpeed(double speed_ms);      // set the sampling speed in ms

public:
   // functins that configure the IO device
   BOOL SetAlarm(const CDigitalChannel& Channel, int AlarmValue);// !NOTE! does not activate the alarm (see ActivateAlarm)
   BOOL ActivateAlarm(const CDigitalChannel& Channel)             {return(LowLevelActivateAlarm(Channel, true));};
   BOOL DeactivateAlarm(const CDigitalChannel& Channel)           {return(LowLevelActivateAlarm(Channel, false));};
   void SetLineAsInput(const CDigitalChannel& Channel)            {LowLevelSetLineIO(Channel, true);};
   void SetLineAsOutput(const CDigitalChannel& Channel)           {LowLevelSetLineIO(Channel, false);};
   void SetOutputData(const CDigitalChannel& Channel, bool Value);
   BOOL InvertIO(const CDigitalChannel& Channel)                  {return(LowLevelInvert(Channel, true));};
   BOOL UnInvertIO(const CDigitalChannel& Channel)                {return(LowLevelInvert(Channel, false));};
   void SetAsShaftEncoder(const CDigitalChannel& Channel)         {LowLevelShaft(Channel, true);};
   void SetAsCounter(const CDigitalChannel& Channel)              {LowLevelShaft(Channel, false);};
   BOOL ConfigureFilters(const CDigitalChannel& Channel, int FilterValue);
   BOOL SetSensitivityHigh(const CDigitalChannel& Channel)        {return LowLevelSetSensitivity(Channel, true);};
   BOOL SetSensitivityLow(const CDigitalChannel& Channel)         {return LowLevelSetSensitivity(Channel, false);};
   BOOL AlarmOnSingleEdge(const CDigitalChannel& Channel)         {return LowLevelAlarmOnEdge(Channel, true);};
   BOOL AlarmOnBothEdges(const CDigitalChannel& Channel)          {return LowLevelAlarmOnEdge(Channel, false);};
   // the above config commands make changes to local variables only
   // in order for the changes to take effect, call Configure (which sends CmdConfigure)
   I2CCODE Configure(bool bForce = false);

public:
   const TCHAR* GetCmdStr(BYTE CmdOpCode);        // returns the string name of the cmd

public:
   // functions that drive a pulse out on a channel for a set time period
   BOOL SetPulseHigh(const CDigitalChannel& Channel)          {return(LowLevelSetPulse(Channel, true));};     // prepare a channel to pulse
   BOOL SetPulseLow(const CDigitalChannel& Channel)                   {return(LowLevelSetPulse(Channel, false));};
   // the above SetPulse commands make changes to local variables only
   // be sure to call PulseOut to acctually drive an output
   I2CCODE PulseOut(double time_ms);               // cause the chanells set by SetPulse to output for time miliseconds

public:
   // functions that configure the IO device
   BOOL LowLevelActivateAlarm(const CDigitalChannel& Channel, bool On);
   void LowLevelSetLineIO(const CDigitalChannel& Channel, bool Input);
   BOOL LowLevelInvert(const CDigitalChannel& Channel, bool Invert);
   void LowLevelShaft(const CDigitalChannel& Channel, bool Shaft);
   BOOL LowLevelSetSensitivity(const CDigitalChannel& Channel, bool High);
   BOOL LowLevelAlarmOnEdge(const CDigitalChannel& Channel, bool SingleEdge);

private:
   BOOL ConfigurationChanged;  // keeps track whether changes were made to the configuration
   BOOL FiltersChanged;
   BYTE FilterUpdateMask;
   BOOL SensitivityChanged;

   IOCONFIGURATION     DeviceConfiguration; // this one is meant to hold the config info of the board as the board supports a write config but not a read config command

public:
   // functions that drive a pulse out on a channel for a set time period
   BOOL LowLevelSetPulse(const CDigitalChannel& Channel, bool Value);
   I2CCODE LowLevelPulseOut(UINT16 KeithTime);
   // KeithTime = 1/32767 sec = 1/0x8000h;  0x21 units KeithTime = 1 ms
private:
   PULSECONFIGURATION PulseConfig;

private:
   // initalization functions
   void DefaultConfiguration(IOCONFIGURATION& config); // init configuration to defaults
   void Initialize();                                  // init our variables etc - constructor wannabe
};

class DECL DigitalIO : public DigitalIO_API
{
public:
   DigitalIO(BYTE AddrOfDevice = 0x04, BYTE AddrOfI2C = 0xFE);
   ~DigitalIO();

public:
   I2CCODE ReadInput(const CDigitalChannel& Channel, bool Invert, bool& Data); // reads & returns input state of channel
   I2CCODE ReadFrequency(const CDigitalChannel& Channel, int Period, bool TakeTwoReadings, double& Data);

private:
   UINT32 LastCount[8];
   UINT32 LastTime[8];
   UINT32 LastTick[8];
};


#endif // !defined(AFX_DigIO_H__65E6C3BC_87B9_11D3_8080_00105A1E5D53__INCLUDED_)
