// DisplayIO.h : main header file for the DisplayIO DLL
//

#if !defined(AFX_DisplayIO_H__65E6C3BC_87B9_11D3_8080_00105A1E5D53__INCLUDED_)
#define AFX_DisplayIO_H__65E6C3BC_87B9_11D3_8080_00105A1E5D53__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#ifndef __AFXWIN_H__
#error include 'stdafx.h' before including this file for PCH
#endif

#include "I2CDeviceClass.h"
#include "I2CMgr.h"
#include "Module.h"
#include "MyQue.h"
#include "Resource.h"
#include "Utils/Report.h"

#undef DECL
#ifdef ENGINE_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif


const int NUM_LINES = 2;
const int NUM_CHARS = 20;
const UCHAR DISP_ESC = 27;


/**********************************************************************************************/

//                                                                           I2C DISPLAY 

/**********************************************************************************************/

// UNRESOLVED - error codes

//this is a device class for handling events from modules
class I2CDISPLAYEVENT : public I2CDEVICE
{
public:
   I2CDISPLAYEVENT(){};
   void Notify(const I2CMSG& Msg);
   MYQUE<I2CMSG> MsgQ;
};

class DECL DisplayIO : public IODevice
{
public:
   DisplayIO(BYTE AddrOfDevice = 0x04, BYTE AddrOfI2C = 0xFE);
   ~DisplayIO();

public:
   // the commands are specific to the Display IO
   I2CCODE ClearDisplay()       {Report.Debug(_T("ClearDisplay")); return Tx(CLEARDISPLAY_OPCODE);};
   I2CCODE DisplayOff()         {Report.Debug(_T("DisplayOff")); return Tx(STOP_OPCODE);};
   I2CCODE HideCursor()         {Report.Debug(_T("HideCursor")); return Tx(HIDECURSOR_OPCODE);};
   I2CCODE StopBlinkingCursor() {Report.Debug(_T("StopBlinkingCursor")); return Tx(STOPBLINKINGCURSOR_OPCODE);};
   I2CCODE ShowCursor(BYTE LineNum, BYTE CurPosition);
   I2CCODE StartBlinkingCursor(BYTE LineNum, BYTE CurPosition);
   I2CCODE WrStringToLCD(bool bClrB4Wr, BYTE byLineNum, BYTE byCurPosition, LPCTSTR szDispString, bool bCenter = true);
   enum KeyStatus{NOACTIVITY, PRESSED, HOLD, RELEASED};
   I2CCODE GetKeyStatus(BYTE LeftKey, BYTE StarKey, BYTE RightKey);
   I2CCODE DisplayLines(LPCTSTR szLine1, LPCTSTR szLine2, bool bCenter = true);
public:
   // High level commands.
   enum EditStatus{EDIT_OK, EDIT_CANCEL, EDIT_TIMEOUT};
   bool OKCancel(bool bDefaultOK = true);
   void Write(LPCTSTR sz, int iLine = 0, bool bCentered = true);
   void Error(LPCTSTR szError);
   EditStatus EditInteger(int& iVal, CString Label, bool bCheck = false, int iLowLim =0, int iHiLim=0);
   EditStatus EditFloat(double& dVal, CString Label, int iRightDigits = 4, bool bCheck = false, double dLowLim =0.0, double dHiLim=0.0);
   EditStatus EditHEX(CString& szStr, CString Label);
   EditStatus EditString(CString& szStr, CString Label, LPCTSTR szCharSet = NULL);
   EditStatus EditTime(CString& szStr, CString Label);
   EditStatus SelectList(LPCTSTR szList[], int iNumItems, int&idx);
public:
   const TCHAR* GetCmdStr(BYTE CmdOpCode);        // returns the string name of the cmd
   I2CDISPLAYEVENT EventDevice;     //tracks channel and module to call for an event msg

public:
   bool KeyPressed();
   void Flush() {EventDevice.MsgQ.Flush();};
   bool Read(char& ch);
   void ResetTimeout() {m_bTimedOut = false;}
   bool TimedOut() {return m_bTimedOut;}
   void SetTimeout(long lTimeout) {m_lTimeout = lTimeout;}
   long GetTimeout() {return m_lTimeout;}
   void ResetReadItr() {ResetEvent(m_hStopEvent);}
   void SetReadItr() {SetEvent(m_hStopEvent);}
private:
   void ChangeChrAtCursor(CString& StrToEdit, int CursorPosition, CString ValidChars, 
         bool bNumber = true);
protected:
   HANDLE m_hStopEvent; // Event handle used to interrupt a read.
   bool m_bTimedOut;    // Keeps track if we have timed out on the display.
   long m_lTimeout;     // Inactivity timeout in ms for the display.

};

#endif // !defined(AFX_DisplayIO_H__65E6C3BC_87B9_11D3_8080_00105A1E5D53__INCLUDED_)
