#ifndef RemoteOps_H
#define RemoteOps_H

#include "Utils/Socket.h"
#include "Utils/Ssp.h"

#undef DECL
#ifdef ENGINE_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif

// Must match the SSPMATCH class in SSPMGR.H, but didn't really want to drag all that code over.
class OPMATCH
{
public:
   OPMATCH(UINT32 SeqMatch, TProtoOpcode OpMatch, UINT32 DataMatch)
      {
         Seq  = SeqMatch;
         Op   = OpMatch;
         Data = DataMatch;
      };
   OPMATCH()
      {
         Seq  = 0;
         Op   = OpEOT;
         Data = OpEOT;
      };
public:
   UINT32 Seq;
   TProtoOpcode Op;
   UINT32 Data;
};


class TServer : public TSocketServer
{
public:
   TServer();
   ~TServer();
   void ServerThread(TServerThread& Thread, TSocket& Socket);
};

class DECL TRemoteOps
{
public:
   TRemoteOps();
   ~TRemoteOps();
   SOCKET LockClient(TSSPData& Command);
   void UnLockClient(SOCKET Match);
private:
   TServer Server;
};

// Send an SSP message to a Com Port controlled by remote, then waits for a response which
// match a set of possible different opcodes and returns the message.
// Note: A response to a RemoteRequest() is marked as "handled" and won't trigger the 
// default parser to process the message. Which is typically preferable, since the default
// parser will often just NAK something it doesn't expect. 
extern "C" DECL int RemoteRequest(int PortIndex, 
   CString Send_To, CString Send_From, DWORD Send_SeqNum, TSSPData& Send_Data, 
   int Count, OPMATCH Match[], 
   CString& Reply_To, CString& Reply_From, DWORD& Reply_SeqNum, TSSPData& Reply_Data,
   int Retries=-1, int AckDelay=-1, DWORD Timeout=120000);

// Sends an SSP message to a Com Port controlled by remote.
extern "C" DECL bool RemoteSend(int PortIndex, 
   CString Send_To, CString Send_From, DWORD Send_SeqNum, TSSPData& Send_Data, 
   DWORD Timeout=120000);

// Waits for and returns an SSP message matching a set of possible different
// opcodes on the specified com port controlled by Remote.
// Note: A response to a WaitMessage() is marked as "handled" and won't trigger the 
// default parser to process the message. Which is typically preferable, since the default
// parser will often just NAK something it doesn't expect. 
extern "C" DECL int RemoteWaitMessage(int PortIndex,
   CString Wait_To, CString Wait_From, 
   int Count, OPMATCH Match[],
   CString& Reply_To, CString& Reply_From, DWORD& Reply_SeqNum, TSSPData& Reply_Data,
   int ReplyDelay=-1, DWORD Timeout=120000);

// Get remote's Port List returning the results in an array of CStrings[9]
// for instance if remote was launched with "remote.exe com1: radio3: ssp6:"
// then StrArray[0]=COM1:, StrArray[2]=RADIO3:, StrArray[5]=SSP6:,
// and the other entries would be blank.
extern "C" DECL bool GetRemotePortList(CString StrArray[]);

// Restart remote with new parameters
extern "C" DECL bool RemoteRestart(CString CommandLine);

////////////////////////////////////////////////////////////////////////////
//The global RemoteOps object, used by application

extern "C" DECL TRemoteOps* RemoteOps;


#endif // RemoteOps_H