// LogDesc.h: interface for the CLogDesc class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_LOGDESC_H__D359BAAE_44F4_11D3_806B_00105A1E5D53__INCLUDED_)
#define AFX_LOGDESC_H__D359BAAE_44F4_11D3_806B_00105A1E5D53__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "..\engine\engine.h"
#include "..\engine\xmlsetup.h"
#include "..\logger\logger.h"

#undef DECL
#ifdef LOGMGR_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif

//used to hold cached data
typedef struct
{
   TCHAR Sensor[129];
   TCHAR Units[129];
   TCHAR Data[1001];
   TCHAR Time[129];
   TCHAR Quality[129];
   TCHAR Date[129];

} LVDATAITEM;
typedef LVDATAITEM * PLVDATAITEM;


////////////////////////////////////////////////////////////////////////////
//this is a class that wraps around the LOG class and allows
//us to track the name, size, and wrap flag for UI purposes
class DECL CLogDesc
{
public:
#ifdef TESTLOG
   void TestLog(CListCtrl * pListCtrl);   //test only
#endif
   void EraseData();          //removes all entries from log
   LOGCURSOR CreateCursor();  //creates a LOGCURSOR object
   bool DumpLog();            //sends log data to serial port
   CLogDesc();                //default constructor
   CLogDesc(TCHAR * lpszName, int Size, bool Wrap, bool IgnoreBadData);
   CLogDesc(LPCTSTR lpszName, int Size, bool Wrap, bool IgnoreBadData);
   ~CLogDesc();
   bool Create();             //creates the actual log file
   LPTSTR GetName(){return name;};
   void SetName(LPCTSTR lpszName) {lstrcpy(name, lpszName);};
   int GetSize(){return size;};
   bool IsWrap(){return wrap;};
   bool IsIgnoreBadData() {return ignorebaddata;};
   // Start and end times are ignored when 0 (results in whole log).
   bool ExportLog(LPCTSTR szFile, int iFormat, DWORD Socket = SOCKET_ERROR, 
         CTime tStart = 0, CTime tEnd = 0);

   enum enumExportFormat {COMMA_DELIMITED, TAB_DELIMITED, 
         FIXED_FORMAT, NUM_EXPORT_FORMATS};
   enum enumExportSpan {WHOLE_LOG, SINCE_LAST_EXPORT, 
         START_TIME_TO_END_TIME, START_TIME_TO_END_LOG, 
         START_LOG_TO_END_TIME, NUM_SPAN_OPTIONS};
   CTime m_tLastExportEnd;

   //to edit properties of log from control panel tree
   int EditProperties(CWnd * pParentWnd, bool bNew = false);
   HTREEITEM hItem;           //handle to this item in control panel tree

   bool InUse();
   BOOL WriteSetup(CXMLSetup& XMLSetup);
   BOOL ReadSetup(CXMLSetup& XMLSetup, CXMLToken& XMLToken);

   //ways to write sensor data to log
   BOOL AppendSensor(CString Sensor, CSensorData& data);
   bool Open();
   LOG log;                   //the log file object itself - see logger.h
   UINT32 ChangeCounter;      //used by logger to track changes

   bool AppendNote(LPCTSTR szNote);
private:
   TCHAR name[256];           //log name
   int size;                  //size of file
   bool wrap;                 //true if it should wrap
   bool ignorebaddata;        //true if we should ignore bad data
};

//////////////////////////////////////////////////////////////////////////
//This class manages the list of log files for the system
class DECL CLogList
{
public:
   void FindLogs();           //finds all files in flash with LOG extension
   CLogDesc * CreateNewLog(CWnd * pParentWnd, LPCTSTR pszName = NULL, bool bShowProps = true);
   void Dump();               //dumps log data out debug port
   CLogList();
   ~CLogList();

   //for saving/restoring setups
   BOOL WriteSetup(CXMLSetup& XMLSetup);
   BOOL ReadSetup(CXMLSetup& XMLSetup, CXMLToken& XMLToken);

   //for manipulating list
   POSITION AddHead(CLogDesc * pLog){return List.AddHead(pLog);};
   POSITION GetHeadPosition(){return List.GetHeadPosition();};
   CLogDesc * GetNext(POSITION& pos){return List.GetNext(pos);};
   int GetCount(){return List.GetCount();};
   CLogDesc * GetHead(){return List.GetHead();};
   POSITION FindIndex(int nIndex){return List.FindIndex(nIndex);};
   POSITION Find(CLogDesc* pLog){return List.Find(pLog);};
   CLogDesc * GetAt(POSITION pos){return List.GetAt(pos);};
   void RemoveAt(POSITION pos){List.RemoveAt(pos);};
   void RemoveAll(){List.RemoveAll();};
   void Add(CLogDesc * pLog);
   CLogDesc * GetByName(LPCTSTR lpszName);
   void Empty();
   void FlushAll();

   // For working with default log.
   POSITION m_posDefault;
   CLogDesc* GetDefault();
   void SetDefault(CLogDesc* pLog) {m_posDefault = Find(pLog);};
   void SetDefault(LPCTSTR lpszName) {m_posDefault = Find(GetByName(lpszName));};

private:
   CList<CLogDesc*, CLogDesc*> List;       //the list itself
};

extern "C" DECL CLogList LogList;

#endif // !defined(AFX_LOGDESC_H__D359BAAE_44F4_11D3_806B_00105A1E5D53__INCLUDED_)
