#ifndef _Report_H_
#define _Report_H_

#undef DECL
#ifdef UTILS_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif

#include "Socket.h"

#ifdef TRACEON
#define TRACELINE Report.Debug(_T("TRACE: %s %d"), _T(__FILE__), __LINE__);
#else
#define TRACELINE ;
#endif

class DECL TReport
{
public:
   TReport();
   ~TReport();

   void Debug(LPCTSTR Fmt, ...);                // A message only output when debugging
   void Status(LPCTSTR Fmt, ...);               // A status message
   void Warning(LPCTSTR Fmt, ...);              // A warning message
   void Error(LPCTSTR Fmt, ...);                // An error message  
   void Fatal(LPCTSTR Fmt, ...);                // A fatal error message
   void Note(LPCTSTR Fmt, ...);                 // User generated note.
   void Maintenance(LPCTSTR Fmt, ...);          // User manually changed an operating parameter
   void SSP(LPCTSTR Fmt, ...);                  // SSP related message
   void I2C(LPCTSTR Fmt, ...);                  // I2C related message

public:
   enum TMsgLevel {msg_All=-1,         msg_Debug=1,      msg_Status=2, msg_Warning=4,
                   msg_Error=8,        msg_Fatal=16,     msg_Note=32, 
                   msg_Maintenance=64, msg_SSP=128,      msg_I2C=256,  msg_None=0};

   // TMsgFunc is the format for a routine which can take over message reporting.
   typedef void (* TMsgFunc)(LPVOID Info, int Level, LPCTSTR Msg);

   void Hook(TMsgFunc MsgFunc, LPVOID Info);    // Hook is used to add a user defined error reporting function
   void UnHook(TMsgFunc MsgFunc);               // UnHook removes a hooked error reporting function

   void SetSerialDebugFilter(int Level);  // Controls which message go out the serial debug port
   int GetSerialDebugFilter();

   void SetFilter(TMsgFunc MsgFunc, int Level); // Sets up a filter on callback hook
   int GetFilter(TMsgFunc MsgFunc);             // Get the filter for a specific hook
   int GetFilter();                             // Get the overall filter based on all hooks
   bool GetRemoteFileVersion(CString& Version); // Get the version of remote that's running
   void StartWatchDog();                        // Start Remote's watchdog
   void StopWatchDog();                         // Stop Remote's watchdog
   void SetBypass(bool State);                  // Enable/Disable direct debug output bypassing Remote
private:
   struct THook
   {
      TMsgFunc Func;
      PVOID Info;
      int Filter;
      THook* Next;
   };

private:
   void DefaultHandler(int Level, LPCTSTR Msg);
   int SerialDebugFilter;
   int Filter;
   bool BypassRemote;
   bool WatchDogOn;
   bool GetConnected();
   THook* HookList;
   CRITICAL_SECTION Lock;
   TSocketClient Remote;
};

extern "C" DECL TReport Report;

#endif // _Report_H_
