//
// Users.h
//

#ifndef _Users_H_
#define _Users_H_


#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#ifndef __AFXWIN_H__
   #error include 'stdafx.h' before including this file for PCH
#endif

#include <afxtempl.h>         // For CTypedPtrArray

#undef DECL
#ifdef UTILS_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif


/*///////////////////////////////////////////////////////////////////////////

1. An instance of the class CUsers is used to contain the set of users and 
   their associated login attributes. The data is loaded from the user data 
   file on construction. The data is written back to file on destruction.
2. Instances of the class CUser are used by CUsers to contain single user 
   accounts. 
3. The user data file is a text file containing a single user account per 
   line. Each account has four fields: name, password, user-group, and timeout.
   The field delimiter is the character '|'. The character '#' is used to 
   indicate no password. Only characters present on the built-in keypad are 
   valid for use in names and passwords.
4. GetUser(CUser& u, int i) and GetUserCount() are subject to filtering based 
   on user group as set by SetFilter(). Note that the index supplied to 
   GetUser() is relative to filtered objects, e.g., GetUser(2) returns the 
   second user found subject to any applied filter. Note that the filter logic 
   is exclusive. For example, a filter of DATA_RETRIEVAL_MODE would filter out 
   all users except those with a user group of DATA_RETRIEVAL_MODE. 

///////////////////////////////////////////////////////////////////////////*/


///////////////////////////////////////////////////////////////////////////////
// TUserGroup - enum used to identify the current login mode CModPropPage
// accessibility. Note that CModPropPage accessibility is cumulative, such that
// higher mode values imply accessibility to modes of lower value.
//
// WARNING: This structure is used by Remote.cpp and XTermDlg.cpp to support 
//  custom groups and access level on login, so try not to do anything that would 
//  change the value of an existing group. If such a change is required be sure 
//  to update Remote.cpp as well and understand upwards compatibility will be 
//  sacrificed.
enum TUserGroup {DATA_RETRIEVAL_MODE, SETUP_MODE, 
      CUSTOM_GROUP1, CUSTOM_GROUP2, CUSTOM_GROUP3, CUSTOM_GROUP4, CUSTOM_GROUP5, 
      CUSTOM_GROUP6, CUSTOM_GROUP7, CUSTOM_GROUP8, CUSTOM_GROUP9, CUSTOM_GROUP10, 
      NUM_USER_GROUPS, NUM_CUSTOM_GROUPS=NUM_USER_GROUPS-CUSTOM_GROUP1};

extern "C" DECL TCHAR* szUserGroups[NUM_USER_GROUPS];

typedef bool (* LPFNCUSTOMLOGIN)(TUserGroup Group, LPCTSTR ComPort, LPCTSTR UserName);
typedef bool (* LPFNCUSTOMCOMMANDPARSER)(TUserGroup Group, LPCTSTR ComPort, LPCTSTR UserName, LPCTSTR Command);

bool DECL AddCustomGroup(TUserGroup Group, LPCTSTR Description, LPFNCUSTOMLOGIN LoginFunct);

bool DECL CallCustomGroup(LPCTSTR UserName, LPCTSTR Password, LPCTSTR ComPort);

bool DECL AddCustomCommandParser(TUserGroup Group, LPFNCUSTOMCOMMANDPARSER CommandParser);

bool DECL CallCustomCommandParser(LPCTSTR UserName, LPCTSTR ComPort, LPCTSTR Command);

extern "C" DECL CString RemoteUserName;
extern "C" DECL CString RemotePassword;
extern "C" DECL DWORD RemoteLoginTime;


/////////////////////////////////////////////////////////////////////////////
// CUser

class DECL CUser : public CObject
{
   friend class CUsers;
   CString m_strName;
   CString m_strPassword;
   TUserGroup m_ugGroup;
   unsigned int m_nTimeoutInterval;

   // Methods
   public:
   CUser& operator=(const CUser& u);
   CString GetName() { return m_strName; };
   CString GetPassword() { return m_strPassword; };
   TUserGroup GetEffectiveUserGroup() { if (m_ugGroup >= CUSTOM_GROUP1) return DATA_RETRIEVAL_MODE; return m_ugGroup; };
   TUserGroup GetUserGroup() { return m_ugGroup; };
   unsigned int GetTimeoutInterval() { return m_nTimeoutInterval; };
};


/////////////////////////////////////////////////////////////////////////////
// CUsers

class DECL CUsers
{
   // Constructors and destructor
   public:
   CUsers();
   ~CUsers();

   // Attributes
   private:
   CFile m_file;
   CTypedPtrArray<CObArray, CUser*> m_Users;
   bool m_bFilter;
   TUserGroup m_ugFilter;
   bool m_bSaveNeeded;

   // Methods
   public:
   bool Add(const CUser& user);
   bool Add(const CString& strName, const CString& strPassword, 
         TUserGroup ug, unsigned int nTimeout);
   bool Remove(const CString& strUser);
   bool SetFilter(TUserGroup ug);
   void RemoveFilter();
   bool GetUser(CUser& u, int i);
   bool GetUser(CUser& u, CString strUser);
   bool UpdateUser(const CString& strUser, const CString& strName, 
         const CString& strPassword, TUserGroup ug, unsigned int nTimeout);
   int GetUserCount();
   bool IsValidUserName(const CString& str);
   bool IsValidPassword(const CString& str);
   bool IsValidUserGroup(int ug);
   bool Commit();
};


#endif //_Users_H_

