#ifndef _SOCKET_H

#define _SOCKET_H

#include <Winsock.h>
#include "ssp.h"

#undef DECL
#if defined(_WINDLL) || defined(XPERT_EXE)
#ifdef UTILS_DLL
#define DECL _declspec(dllexport)
#else
#define DECL _declspec(dllimport)
#endif
#else
#define DECL 
#endif


const SOCKET_NULL = SOCKET_ERROR;

class DECL TThread
{
public:
   TThread();
   virtual ~TThread();
   virtual void ThreadLoop() {};
   HANDLE hStopThread;
   HANDLE hThreadStopped;
   bool IsDone();
private:
   HANDLE hThread;
   friend DWORD TThread_BootStrap(TThread* Thread);
};


class DECL TSocket
{
public:
   TSocket();
   TSocket(SOCKET aSocket);
   virtual ~TSocket();
// Character Read/Write with Timeouts
   int Read(char* Buffer, int MaxLen, int Timeout=0);
   int Write(const char* Buffer, int Len, int Timeout=0);
// SSP Read/Write
   bool Read(TSSPData& SSPData, int Timeout);
   bool Write(TSSPData& SSPData, int Timeout);
//
   void DisableBuffer();
   int Abort();
   int Reset();
   int Close(bool Gracefull=false);
   bool Closed();
   bool ReadyToWrite(DWORD Timeout);
   int GetInputCount(DWORD Timeout = 0);
   SOCKET SetSocket(SOCKET NewSocket);
   SOCKET GetSocket();
   void FlushInput();
   bool GetSockName(CString& Address, int& Port);
private:
   friend class TSocketClient;
   friend class TSocketServer;
   SOCKET Socket;
};

class DECL TServerThread : public TThread
{
public:
   TServerThread(TSocketServer& ASocketServer, int ASocket);
   virtual ~TServerThread();
   void ThreadLoop();
   bool IsDone();
public:
   TSocket Socket;
   HANDLE RequestLoan;
   HANDLE GrantLoan;
   HANDLE DoneLoan;
   bool LoanApproved;
   bool Done;
private:
   friend class TSocketServer;
   TSocketServer& SocketServer;
};

class DECL TSocketServer
{
public:
   TSocketServer();
   virtual ~TSocketServer();
   virtual void ServerThread(TServerThread& Thread, TSocket& Socket) {};
   virtual TServerThread* CreateServerThread(int ASocket);
   int StartServer(int aPort);
   int StopServer();
   int GetPort();
   bool IsDone();
public:
   CPtrArray ServerList;
   CRITICAL_SECTION Lock;
   void FreezeServerList();
   void UnFreezeServerList();
   int Port;
private:
   int FreezeCount;
   friend DWORD TSocketServer_BootStrap(TSocketServer* SocketServer);
   friend class TServerThread;
   TSocket ListenSocket;
   HANDLE hListenThread;
   void ListenThread();
   bool Done;
};

class DECL TSocketClient : public TSocket
{
public:
   TSocketClient();
   virtual ~TSocketClient();
   int Connect(LPCTSTR Addr, int Port);
   bool IsConnected();
};


/*
   TSocketClient Test Code
   -----------------------

   char Buffer[10000];
   TSocketClient Client;
   int ConRes = Client.Connect(_T("www.ibm.com"), 80);
   int WError = WSAGetLastError();
   int WrRes = Client.Write("GET /index.html\r\n", 17, 5000);
   int ReadLen = Client.Read(&Buffer[0], 10000, 5000);
   Client.Close();

*/

#endif // _SOCKET_H